/*
  thorfinn@binf.ku.dk april 15 2012
  class to call genotypes
 */

#include "shared.h"
#include <cmath>
#include "analysisFunction.h"

//filename of dumped file
#define GENO ".geno"

//stuff related to genotypecalling
#define GENO_MAJOR_MINOR 0x1 // write the major and the minor
#define GENO_MAF_PRIOR 0x2 //print the called genotype
#define GENO_TAKE_MAX 0x4 //just pick the maxim likelihood
#define GENO_WRITE_POST 0x8 //write the post of the called genotype
#define GENO_ALL_POST 0x10 // write the post
#define GENO_WRITE_DEPTH 0x20 //write the depth
#define GENO_FOR_COVAR 0x40 //binary dump of the posteriors.




class callGenotypes:public general{
public:
  int doGeno;
  int doMaf;
  //none optional stuff
  FILE *outfile;
  callGenotypes(const char *outfiles,argStruct *arguments,int inputtype);
  ~callGenotypes();
  void getOptions(argStruct *arguments);
  void run(funkyPars  *pars);
  void clean(funkyPars *pars);
  void print(funkyPars *pars);
  void printArg(FILE *argFile);

};
void callGenotypes::printArg(FILE *argFile){
  fprintf(argFile,"%s:\n\n",__FILE__);
  fprintf(argFile,"-doGeno\t%d\n",doGeno);
  fprintf(argFile,"\t->1: Thorfinn add something\n");
  fprintf(argFile,"\t->2: Thorfinn add something\n");
  fprintf(argFile,"\n");

}


void callGenotypes::getOptions(argStruct *arguments){
  //default
  doMaf=0;
  doGeno=0;
  //from command line
  doGeno=angsd::getArg("-doGeno",doGeno,arguments);
 if(doGeno==-999){
    doGeno=0;
    printArg(stdout);
    exit(0);
   }
 
  if(doGeno==0)
    return;
  doMaf=angsd::getArg("-doMaf",doMaf,arguments);
  printArg(arguments->argumentFile);

}

callGenotypes::callGenotypes(const char *outfiles,argStruct *arguments,int inputtype){

  getOptions(arguments);

  if(doGeno==0)
    return;

  //inputtype 0) soap 1) samglf 2) samglfClean 3) tglf 4)sim type 5) beagle
  if((doGeno&GENO_WRITE_POST||doGeno&GENO_FOR_COVAR || doGeno&GENO_ALL_POST)&& (doGeno&GENO_MAF_PRIOR)!=0) {
    fprintf(stderr,"\n\t-> For dumping post in genotypecalling you need the MAF-model\n\tsetting mafprior in genotypecaller \n\n");
    doGeno =     doGeno|GENO_MAF_PRIOR;
    fflush(stderr);
    //    exit(0);
  }

  if((doGeno&GENO_MAF_PRIOR ) &&(doMaf==0)){
    fprintf(stderr,"\n\t-> You need a maf estimator for this genotypecaller\n\t\n\n");
    //    args->doMaf = args->doMaf|MAF_KNOWN_EM;
    fflush(stderr);
    exit(0);
  }

  
  

  fprintf(stderr,"Calling genotypes: %s\n",outfiles);
  //make output files
  const char* postfix;
  postfix=GENO;
  outfile = openFile(outfiles,postfix);
  FILE *oFileGenos=outfile;
  if(!(doGeno&GENO_FOR_COVAR)){
    fprintf(oFileGenos,"chromo\tposition\t");
    if(doGeno&GENO_MAJOR_MINOR)
      fprintf(oFileGenos,"major\tminor\t");
    if(doGeno&GENO_MAF_PRIOR)
      fprintf(oFileGenos,"HWEPRIOR\t");
    if(doGeno&GENO_TAKE_MAX)
      fprintf(oFileGenos,"MAXLRT\t");
    if(doGeno&GENO_WRITE_POST)
      fprintf(oFileGenos,"post1\t");
    if(doGeno&GENO_ALL_POST)
      fprintf(oFileGenos,"ALLpost3\t");
    if(doGeno&GENO_WRITE_DEPTH)
      fprintf(oFileGenos,"depth\t");
    fprintf(oFileGenos,"\n");
  }
  //print header
  
}


callGenotypes::~callGenotypes(){
  //  fprintf(stderr,"[%s] calling destructor\n",__FUNCTION__);
  //  fflush(stderr);
  if(doGeno==0)
    return;

  fclose(outfile);
}

int which3Max(double *offVal){
  if(offVal[0]>offVal[1] && offVal[0] >offVal[2])
    return 0;
  else if(offVal[1]>offVal[0] && offVal[1] >offVal[2])
    return 1;
  else if(offVal[2]>offVal[1] && offVal[2] >offVal[0]){
    return 2;
  }else
      fprintf(stderr,"floating point problem in function: %s\n",__FUNCTION__);

}


void getGenos(int *hwe,int *whichmax,double *post,double *lik,double maf,int nInd,int maj,int min,int all3Posts){
  //deleteme start
  /*
  for(int i=0;i<10*nInd;i++)
    fprintf(stderr,"%f,",lik[i]);
  fprintf(stderr,"\n");
  */
  //delete me stop
  for( int i =0;i<nInd;i++){
    int s;
    for(s=1;s<10;s++)
      if(lik[i*10]!=lik[i*10+s])
	break;
    if(s==9){//this should be 10...
      hwe[i] = -1;
      whichmax[i] = -1;
      post[i] = 0;
      fprintf(stderr,"here\n");//deleteme
      continue;
    }
    //    fprintf(stderr,"s=%d\n",s);
    //find max id with maxlike
    int mmax=0;
    for(s=1;s<10;s++)
      if(lik[i*10+s]>lik[i*10+mmax])
	mmax = s;
    whichmax[i] = mmax;
    
    int offID[3] = {angsd::majorminor[maj][maj],angsd::majorminor[maj][min],angsd::majorminor[min][min]};
    double offVal[3];
    offVal[0] = exp(lik[i*10+offID[0]])*pow(1.0-maf,2.0);
    offVal[1] = exp(lik[i*10+offID[1]])*2*maf*(1-maf);
    offVal[2] = exp(lik[i*10+offID[2]])*pow(maf,2.0);
    double ssum = offVal[0]+offVal[1]+offVal[2];
    hwe[i] = offID[which3Max(offVal)];
    if(all3Posts){
      for(int n=0;n<3;n++)
	post[i*3+n] = offVal[n]/ssum;
    }else
      post[i] = offVal[which3Max(offVal)]/ssum;

  }
}



void printDoGeno( int *keepSites, int**depth,double **loglikes,loci *sites,int nInd,int nSites,int doGeno,double *MAF,FILE *fp,int *major,int *minor) {
  //  fprintf(stderr,"doGeno=%d\n",doGeno);
  if(keepSites==NULL){
    fprintf(stderr,"\t->[%s] We would expect to have an array of keepsites\n",__FUNCTION__);
    exit(0);

  }
  //  fprintf(stderr,"%s\t%d\n",sites[0].chromo,sites[0].position);
  //fprintf(stderr,"%s\t%d\n",sites[1].chromo,sites[1].position);  

  for(int s=0;s<nSites;s++) {
    //fprintf(stderr,"s=%d,",s);
    if(keepSites[s]==0){//will skip if all genotypeliklihoods =0
      fprintf(stderr,"Dont keep:%d\n",s);
      continue;
    } 
    if(!(doGeno&GENO_FOR_COVAR))
      fprintf(fp,"%s\t%d\t",sites[s].chromo,sites[s].position+1);
    if(doGeno&GENO_MAJOR_MINOR) //1
      fprintf(fp,"%c\t%c\t",intToRef[major[s]],intToRef[minor[s]]);
    int hwe[nInd];
    int whichmax[nInd];
    int all3Posts = (doGeno&GENO_ALL_POST) + (doGeno&GENO_FOR_COVAR);//we need all 3 genotypes when dumping for covar.IMPORTANT with paranthesis
    double post[nInd*3];//we might only need nInd, but this is on stack so don't care
    getGenos(hwe,whichmax,post,loglikes[s],MAF[s],nInd,major[s],minor[s],all3Posts);
    if(doGeno&GENO_FOR_COVAR){
      fwrite(post,sizeof(double),3*nInd,fp);
      //      exit(0);//deleteme
      continue;
    }

    for(int i=0;i<nInd;i++){
      if(doGeno&GENO_MAF_PRIOR) //2
	fprintf(fp,"%d\t",hwe[i]);
      if(doGeno&GENO_TAKE_MAX)//4
	fprintf(fp,"%d\t",whichmax[i]);
      if(doGeno&GENO_WRITE_POST)//8
	fprintf(fp,"%f\t",post[i]);
      if(doGeno&GENO_ALL_POST)//16
	for(int n=0;n<3;n++)
	  fprintf(fp,"%f\t",post[i*3+n]);
      if((doGeno&GENO_WRITE_DEPTH)&&(depth!=NULL)) //32 we have doepths
	fprintf(fp,"%d\t",depth[s][i]);
      if((doGeno&GENO_WRITE_DEPTH)&&(depth==NULL))//32 don't have depth print NA
	fprintf(fp,"NA\t");
    }
    fprintf(fp,"\n");

  }

}




void callGenotypes::clean(funkyPars *p){
  if(doGeno==0)
    return;
  
  
  
}


void callGenotypes::print(funkyPars *p){
  if(doGeno==0)
    return;
  
  printDoGeno(p->keepSites,p->depth,p->likes,p->sites,p->nInd,p->numSites,doGeno,p->results->asso->freq,outfile,p->major,p->minor);
  
}

//do nothing every thing is precalucated 
void callGenotypes::run(funkyPars *pars){
  
}

